//   Copyright 2012 Georg-August-Universität Göttingen, Germany
//
//   Licensed under the Apache License, Version 2.0 (the "License");
//   you may not use this file except in compliance with the License.
//   You may obtain a copy of the License at
//
//       http://www.apache.org/licenses/LICENSE-2.0
//
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS,
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and
//   limitations under the License.

package de.ugoe.cs.autoquest.commands.usage;

import java.util.List;

import de.ugoe.cs.autoquest.CommandHelpers;
import de.ugoe.cs.autoquest.usageprofiles.FirstOrderMarkovModel;
import de.ugoe.cs.autoquest.usageprofiles.HighOrderMarkovModel;
import de.ugoe.cs.autoquest.usageprofiles.ModelFlattener;
import de.ugoe.cs.util.console.Command;
import de.ugoe.cs.util.console.GlobalDataContainer;

/**
 * <p>
 * Command to flatten high-order models and create first-order markov models
 * with the same stochastic properties.
 * </p>
 * 
 * @author Steffen Herbold
 * @version 1.0
 */
public class CMDflattenModel implements Command {

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.ugoe.cs.util.console.Command#run(java.util.List)
	 */
	@Override
	public void run(List<Object> parameters) {
		String modelname;
		String modelnameFOM;

		try {
			modelname = (String) parameters.get(0);
			modelnameFOM = (String) parameters.get(1);
		} catch (Exception e) {
			throw new IllegalArgumentException();
		}

		Object dataObject = GlobalDataContainer.getInstance()
				.getData(modelname);
		if (dataObject == null) {
			CommandHelpers.objectNotFoundMessage(modelname);
			return;
		}
		if (!(dataObject instanceof HighOrderMarkovModel)) {
			CommandHelpers.objectNotType(modelname, "HighOrderMarkovModel");
			return;
		}

		HighOrderMarkovModel model = (HighOrderMarkovModel) dataObject;
		ModelFlattener flattener = new ModelFlattener();
		FirstOrderMarkovModel modelFOM = flattener
				.flattenHighOrderMarkovModel(model);

		if (GlobalDataContainer.getInstance().addData(modelnameFOM, modelFOM)) {
			CommandHelpers.dataOverwritten(modelnameFOM);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see de.ugoe.cs.util.console.Command#help()
	 */
	@Override
	public String help() {
		return "flattenModel <modelname> <modelname_flattened>";
	}
}
